@@ -1,65 +1,20 @@
 use regex::Regex;
 use thiserror::Error;
-use chrono::{Timelike, Datelike};
+use chrono::Timelike;
 
-use crate::version;
-use crate::constellation;
-use crate::is_rinex_comment;
+use crate::gnss_time;
+use crate::version::RinexVersion;
+use crate::observation::ObservationType;
+use crate::{is_rinex_comment, RinexType, RinexTypeError};
+use crate::constellation::{Constellation, ConstellationError};
 
 /// Describes a `CRINEX` (compact rinex) 
 pub const CRINEX_MARKER_COMMENT : &str = "COMPACT RINEX FORMAT";
 /// End of Header section reached
 pub const HEADER_END_MARKER : &str = "END OF HEADER";
 
-/// Describes all known `RINEX` file types
-#[derive(Copy, Clone, Debug)]
-pub enum RinexType {
-    ObservationData,
-    NavigationMessage,
-    MeteorologicalData,
-    ClockData,
-}
-
-#[derive(Error, Debug)]
-pub enum RinexTypeError {
-    #[error("Unknown RINEX type identifier \"{0}\"")]
-    UnknownType(String),
-}
-
-impl Default for RinexType {
-    /// Builds a default `RinexType`
-    fn default() -> RinexType { RinexType::ObservationData }
-}
-
-impl RinexType {
-    /// Converts `Self` to string
-    pub fn to_string(&self) -> &str {
-        match *self {
-            RinexType::ObservationData => "ObservationData",
-            RinexType::NavigationMessage => "NavigationMessage",
-            RinexType::MeteorologicalData => "MeteorologicalData",
-            RinexType::ClockData => "ClockData",
-        }
-    }
-}
-
-impl std::str::FromStr for RinexType {
-    type Err = RinexTypeError;
-    fn from_str (s: &str) -> Result<Self, Self::Err> {
-        if s.eq("NAVIGATION DATA") {
-            Ok(RinexType::NavigationMessage)
-        } else if s.contains("NAV DATA") {
-            Ok(RinexType::NavigationMessage)
-        } else if s.eq("OBSERVATION DATA") {
-            Ok(RinexType::ObservationData)
-        } else {
-            Err(RinexTypeError::UnknownType(String::from(s)))
-        }
-    }
-}
-
 /// GNSS receiver description
-#[derive(Debug, PartialEq)]
+#[derive(Debug)]
 struct Rcvr {
     model: String, 
     sn: String, // serial #
@@ -71,8 +26,8 @@ impl Default for Rcvr {
     fn default() -> Rcvr {
         Rcvr {
             model: String::from("Unknown"),
-            sn: String::from("?"),
-            firmware: String::from("?"),
+            sn: String::from("Unknown"),
+            firmware: String::from("Unknown"),
         }
     }
 }
@@ -80,11 +35,14 @@ impl Default for Rcvr {
 impl std::str::FromStr for Rcvr {
     type Err = std::io::Error;
     fn from_str (line: &str) -> Result<Self, Self::Err> {
-        let (id, remainder) = line.split_at(20);
-        let (make, version) = remainder.split_at(20);
-        println!("RCVR | ID \"{}\" | MAKE \"{}\" | VERSION \"{}\"",
-            id, make, version);
-        Ok(Rcvr::default())
+        let (id, rem) = line.split_at(20);
+        let (make, rem) = rem.split_at(20);
+        let (version, rem) = rem.split_at(20);
+        Ok(Rcvr{
+            sn: String::from(id.trim()),
+            model: String::from(make.trim()),
+            firmware: String::from(version.trim()),
+        })
     }
 }
 
@@ -94,11 +52,9 @@ struct Antenna {
     model: String,
     sn: String, // serial #
     coords: Option<rust_3d::Point3D>, // ref. point position
-        // for recordings generated onboard a vehicule
-    height: Option<f32>, // height in comparison to 
-        // reference point (ARP) above marker
-    delta_h_e_n: Option<f32>, // horizontal eccentricity (ARP) 
-        // relative to marker (EAST/NORTH) 
+    height: Option<f32>, // height in comparison to ref. point
+    eastern_eccentricity: Option<f32>, // in comparison to ref. point
+    northern_eccentricity: Option<f32>, // in comparison to ref. point
 }
 
 impl Default for Antenna {
@@ -106,109 +62,94 @@ impl Default for Antenna {
     fn default() -> Antenna {
         Antenna {
             model: String::from("Unknown"),
-            sn: String::from("?"),
+            sn: String::from("Unknown"),
             coords: None,
             height: None,
-            delta_h_e_n: None,
+            eastern_eccentricity: None,
+            northern_eccentricity: None,
         }
     }
 }
 
-/// GnssTime struct is a time realization,
-/// and the `GNSS` constellation that produced it
-#[derive(Debug)]
-struct GnssTime {
-    time: chrono::NaiveDateTime,
-    gnss: constellation::Constellation,
-}
-
-impl Default for GnssTime {
-    /// Builds default `GnssTime` structure
-    fn default() -> GnssTime {
-        let now = chrono::Utc::now();
-        GnssTime {
-            time: chrono::NaiveDate::from_ymd(
-                now.date().year(),
-                now.date().month(),
-                now.date().day(),
-            ).and_hms(
-                now.time().hour(),
-                now.time().minute(),
-                now.time().second()
-            ),
-            gnss: constellation::Constellation::default(),
-        }
+impl std::str::FromStr for Antenna {
+    type Err = std::io::Error;
+    fn from_str (line: &str) -> Result<Self, Self::Err> {
+        let (id, rem) = line.split_at(20);
+        let (make, rem) = rem.split_at(20);
+        Ok(Antenna{
+            sn: String::from(id.trim()),
+            model: String::from(make.trim()),
+            coords: None,
+            height: None,
+            eastern_eccentricity: None,
+            northern_eccentricity : None,
+        })
     }
 }
 
-impl GnssTime {
-    /// Builds a new `GnssTime` object
-    fn new(time: chrono::NaiveDateTime, gnss: constellation::Constellation) -> GnssTime {
-        GnssTime {
-            time, 
-            gnss
-        }
-    }
+impl Antenna {
+    /// Assigns antenna coordinates
+    pub fn set_coords (&mut self, coords: rust_3d::Point3D) { self.coords = Some(coords) }
+    /// Sets Antenna height compared to reference point
+    pub fn set_height (&mut self, height: f32) { self.height = Some(height) }
+    /// Sets Eastern eccentricity (m)
+    pub fn set_eastern_eccentricity (&mut self, e: f32) { self.eastern_eccentricity = Some(e) }
+    /// Sets Northern eccentricity (m)
+    pub fn set_northern_eccentricity (&mut self, e: f32) { self.northern_eccentricity = Some(e) }
 }
 
-/// `LeapSecond` to describe leap seconds
-struct LeapSecond {
-    leap: u32, // current amount of leap secs
+/// `LeapSecond` to describe leap seconds.
+/// leap: current number of leap seconds 
+/// past_future: future or past leap seconds ΔtLS,   
+/// ie., future leap second if week and day number are in future   
+/// week: week counter sometimes present.   
+/// day: week counter sometimes present.   
+/// GnssTimes:   
+/// GLO = UTC = GPS - ΔtLS   
+/// GPS = GPS = UTC + ΔtLS   
+#[derive(Copy, Clone, Debug)]
+pub struct LeapSecond {
+    leap: u32, // current number
+    past_future: u32, // future or past leap seconds ΔtLS   
     week: u32, // week number 
-    day: u32,
-    delta: u32, // ΔtLSF(BNK) [s]
-        // delta time between GPS and UTC due to leap second
-        // can be future or past ΔtLSF(BNK) depending
-        // wether (week,day) are in future or past
-    constellation: constellation::Constellation, // system time identifier
+    day: u32, // day number
 }
 
 impl Default for LeapSecond {
+    /// Builds a default (null) `LeapSecond`
     fn default() -> LeapSecond {
         LeapSecond {
             leap: 0,
+            past_future: 0,
             week: 0,
             day: 0,
-            delta: 0,
-            constellation: constellation::Constellation::default(),
-        }
-    }
-}
-
-impl LeapSecond {
-    // Builds a new leap second
-    pub fn new (leap: u32, week: Option<u32>, day: Option<u32>, delta: Option<u32>,
-        constellation: Option<constellation::Constellation>)
-            -> LeapSecond {
-        LeapSecond {
-            leap: leap,
-            week: week.unwrap_or(0),
-            day: day.unwrap_or(0),
-            delta: delta.unwrap_or(0),
-            constellation: constellation.unwrap_or(constellation::Constellation::GPS),
         }
     }
 }
 
 impl std::str::FromStr for LeapSecond {
-    type Err = HeaderError; 
+    type Err = Error; 
+    /// Builds `LeapSecond` from string descriptor
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         let mut ls = LeapSecond::default();
         // leap seconds might have either simple or complex format
         let items: Vec<&str> = s.split_ascii_whitespace()
             .collect();
-        match items.len() {
-            1 => {
+        match items.len() > 2 {
+            false => {
                 ls.leap = u32::from_str_radix(items[0].trim(),10)?
             },
-            4 => {
-                ls.leap = u32::from_str_radix(items[0].trim(),10)?;
-                ls.week = u32::from_str_radix(items[1].trim(),10)?;
-                ls.day = u32::from_str_radix(items[2].trim(),10)?
-                //ls.constellation = Constellation: //TODO
-                //18    18  2185     7GPS             LEAP SECONDS        
+            true => {
+                let (leap, rem) = s.split_at(6);
+                let (past, rem) = rem.split_at(6);
+                let (week, rem) = rem.split_at(6);
+                let (day, rem) = rem.split_at(6);
+                ls.leap = u32::from_str_radix(leap.trim(),10)?;
+                ls.past_future = u32::from_str_radix(past.trim(),10)?;
+                ls.week = u32::from_str_radix(week.trim(),10)?;
+                ls.day = u32::from_str_radix(day.trim(),10)?
             },
-            _ => return Err(HeaderError::LeapSecondParsingError(String::from(s)))
+            _ => return Err(Error::LeapSecondParsingError(String::from(s)))
         }
         Ok(ls)
     }
@@ -263,33 +204,44 @@ enum SignalStrength {
 
 /// Describes `RINEX` file header
 #[derive(Debug)]
-pub struct Header {
-    version: version::Version, // version description
+pub struct RinexHeader {
+    version: RinexVersion, // version description
     crinex: Option<CrinexInfo>, // if this is a CRINEX
     rinex_type: RinexType, // type of Rinex
-    constellation: constellation::Constellation, // GNSS constellation being used
+    constellation: Constellation, // GNSS constellation being used
     program: String, // program name 
     run_by: String, // program run by
     //date: strtime, // file date of creation
     station: String, // station label
     station_id: String, // station id
+    station_url: Option<String>, // station url
     observer: String, // observer
     agency: String, // agency
-    rcvr: Option<Rcvr>, // receiver used for this recording
+    rcvr: Option<Rcvr>, // optionnal hardware infos
     ant: Option<Antenna>, // optionnal antenna infos
-    leap: Option<u32>, // leap seconds
+    leap: Option<LeapSecond>, // optionnal leap seconds infos
     coords: Option<rust_3d::Point3D>, // station approx. coords
-    wavelengths: Option<(u32,u32)>, // L1/L2 wavelengths
+    wavelengths: Option<(u32,u32)>, // observations wavelengths
     sampling_interval: Option<f32>, // sampling
-    epochs: (Option<GnssTime>, Option<GnssTime>), // first , last observations
+    epochs: (Option<gnss_time::GnssTime>, Option<gnss_time::GnssTime>), // first , last observations
+    license: Option<String>, // optionnal license
+    doi: Option<String>, // optionnal Object Idenfier ("IoT")
+    gps_utc_delta: Option<u32>, // optionnal GPS / UTC time difference
+    // processing
+    scaling_factor: Option<f64>, // optionnal data scaling
+    // optionnal ionospheric compensation param(s)
+    //ionospheric_corr: Option<Vec<IonoCorr>>,
+    // possible time system correction(s)
+    //gnsstime_corr: Option<Vec<gnss_time::GnssTimeCorr>>,
     // true if epochs & data compensate for local clock drift 
     rcvr_clock_offset_applied: Option<bool>, 
-    gps_utc_delta: Option<u32>, // optionnal GPS / UTC time difference
-    sat_number: Option<u32>, // nb of sat for which we have data
+    // observation (specific)
+    // obs_types: lists all types of observations contained in this `Rinex`
+    obs_types: Option<std::collections::HashMap<Constellation, Vec<ObservationType>>>,
 }
 
 #[derive(Error, Debug)]
-pub enum HeaderError {
+pub enum Error {
     #[error("Compact RINEX related content mismatch")]
     CrinexFormatError,
     #[error("RINEX version is not supported '{0}'")]
@@ -299,7 +251,7 @@ pub enum HeaderError {
     #[error("rinex type error")]
     RinexTypeError(#[from] RinexTypeError),
     #[error("unknown GNSS Constellation '{0}'")]
-    UnknownConstellation(#[from] constellation::ConstellationError),
+    UnknownConstellation(#[from] ConstellationError),
     #[error("failed to parse antenna / receiver infos")]
     AntennaRcvrError(#[from] std::io::Error),
     #[error("failed to parse integer value")]
@@ -312,35 +264,45 @@ pub enum HeaderError {
     LeapSecondParsingError(String),
 }
 
-impl Default for Header {
-    fn default() -> Header {
-        Header {
-            version: version::Version::default(), 
+impl Default for RinexHeader {
+    fn default() -> RinexHeader {
+        RinexHeader {
+            version: RinexVersion::default(), 
             crinex: None,
             rinex_type: RinexType::default(),
-            constellation: constellation::Constellation::default(),
+            constellation: Constellation::default(),
             program: String::from("Unknown"),
             run_by: String::from("Unknown"),
             station: String::from("Unknown"),
             station_id: String::from("Unknown"),
             observer: String::from("Unknown"),
             agency: String::from("Unknown"),
-            rcvr: None, 
-            ant: None, 
-            coords: None, 
+            station_url: None,
             leap: None,
-            rcvr_clock_offset_applied: None,
-            wavelengths: None,
-            sampling_interval: None,
-            epochs: (None, None),
+            license: None,
+            doi: None,
             gps_utc_delta: None,
-            sat_number: Some(0)
+            // hardware
+            rcvr: None,
+            // antenna
+            ant: None,
+            coords: None, 
+            // observations
+            epochs: (None, None),
+            wavelengths: None,
+            obs_types: None,
+            // processing
+            rcvr_clock_offset_applied: None,
+            scaling_factor: None,
+            //ionospheric_corr: None,
+            //gnsstime_corr: None,
+            sampling_interval: None,
         }
     }
 }
 
-impl std::str::FromStr for Header {
-    type Err = HeaderError;
+impl std::str::FromStr for RinexHeader {
+    type Err = Error;
     /// Builds header from extracted header description
     fn from_str (content: &str) -> Result<Self, Self::Err> {
         let mut lines = content.lines();
@@ -387,19 +349,19 @@ impl std::str::FromStr for Header {
         let (constellation_str, remainder) = remainder.trim().split_at(20);
 
         let rinex_type = RinexType::from_str(type_str.trim())?;
-        let mut constellation: constellation::Constellation;
+        let mut constellation: Constellation;
         
         if type_str.trim().contains("GLONASS") {
             // special case, sometimes GLONASS NAV
             // drops the constellation field cause it's implied
-            constellation = constellation::Constellation::Glonass
+            constellation = Constellation::Glonass
         } else {
-            constellation = constellation::Constellation::from_str(constellation_str.trim())?
+            constellation = Constellation::from_str(constellation_str.trim())?
         }
 
-        let version = version::Version::from_str(version_str.trim())?;
+        let version = RinexVersion::from_str(version_str.trim())?;
         if !version.is_supported() {
-            return Err(HeaderError::VersionNotSupported(String::from(version_str)))
+            return Err(Error::VersionNotSupported(String::from(version_str)))
         }
 
         // line2
@@ -481,16 +443,26 @@ impl std::str::FromStr for Header {
         let mut station_id = String::from("Unknown");
         let mut observer   = String::from("Unknown");
         let mut agency     = String::from("Unknown");
+        let mut license     : Option<String> = None;
+        let mut doi         : Option<String> = None;
+        let mut station_url : Option<String> = None;
         // hardware
         let mut ant        : Option<Antenna> = None;
         let mut ant_coords : Option<rust_3d::Point3D> = None;
+        let mut ant_hen    : Option<(f32,f32,f32)> = None;
         let mut rcvr       : Option<Rcvr>    = None;
         // other
-        let mut leap       : Option<u32>     = None;
+        let mut leap       : Option<LeapSecond> = None;
         let mut sampling_interval: Option<f32> = None;
         let mut rcvr_clock_offset_applied: Option<bool> = None;
         let mut coords     : Option<rust_3d::Point3D> = None;
-        let mut epochs: (Option<GnssTime>, Option<GnssTime>) = (None, None);
+        let mut epochs: (Option<gnss_time::GnssTime>, Option<gnss_time::GnssTime>) = (None, None);
+        // RinexType::ObservationData
+        let mut obs_nb_sat : u32 = 0;
+        let mut obs_types: 
+            Option<std::collections::HashMap<Constellation, Vec<ObservationType>>> 
+                = None;
+
         loop {
             /*
             <o
@@ -513,57 +485,53 @@ impl std::str::FromStr for Header {
                 rcvr = Some(Rcvr::from_str(line)?) 
             
             } else if line.contains("ANT # / TYPE") {
-                let (id, remainder) = line.split_at(20);
-                let (make, remainder) = remainder.trim().split_at(20);
-                println!("ANTENNA | ID \"{}\" | MAKE \"{}\"", id, make);
+                ant = Some(Antenna::from_str(line)?)
             
             } else if line.contains("LEAP SECOND") {
-                // TODO
-                // LEAP SECOND might have complex format
-                //let leap_str = line.split_at(20).0.trim();
-                //leap = Some(u32::from_str_radix(leap_str, 10)?)
+                leap = Some(LeapSecond::from_str(line.split_at(40).0)?)
 
             } else if line.contains("DOI") {
-                // TODO digital object identifier
-                // v> 4.0
+                let (content, _) = line.split_at(40); //  TODO: confirm please
+                doi = Some(String::from(content.trim()))
+
             } else if line.contains("MERGED FILE") {
-                //TODO nb# of merged files
-                // v>4.0
+                //TODO V > 3 nb# of merged files
+
             } else if line.contains("STATION INFORMATION") {
-                // TODO station URL
-                // v>4
+                let (url, _) = line.split_at(40); //TODO confirm please 
+                station_url = Some(String::from(url.trim()))
+
             } else if line.contains("LICENSE OF USE") {
-                // TODO license in use
-                // v>4
-            
+                let (lic, _) = line.split_at(40); //TODO confirm please 
+                license = Some(String::from(lic.trim()))
             
             } else if line.contains("TIME OF FIRST OBS") {
                 let items: Vec<&str> = line.split_ascii_whitespace()
                     .collect();
-                let (y, month, d, h, min, s, constel): (i32,u32,u32,u32,u32,f32,constellation::Constellation) =
+                let (y, month, d, h, min, s, constel): (i32,u32,u32,u32,u32,f32,Constellation) =
                     (i32::from_str_radix(items[0].trim(),10)?,
                     u32::from_str_radix(items[1].trim(),10)?,
                     u32::from_str_radix(items[2].trim(),10)?,
                     u32::from_str_radix(items[3].trim(),10)?,
                     u32::from_str_radix(items[4].trim(),10)?,
                     f32::from_str(items[5].trim())?,
-                    constellation::Constellation::from_str(items[6].trim())?);
+                    Constellation::from_str(items[6].trim())?);
                 let utc = chrono::NaiveDate::from_ymd(y,month,d).and_hms(h,min,s as u32);
-                epochs.0 = Some(GnssTime::new(utc, constel)) 
+                epochs.0 = Some(gnss_time::GnssTime::new(utc, constel)) 
 
             } else if line.contains("TIME OF LAST OBS") {
                 let items: Vec<&str> = line.split_ascii_whitespace()
                     .collect();
-                let (y, month, d, h, min, s, constel): (i32,u32,u32,u32,u32,f32,constellation::Constellation) =
+                let (y, month, d, h, min, s, constel): (i32,u32,u32,u32,u32,f32,Constellation) =
                     (i32::from_str_radix(items[0].trim(),10)?,
                     u32::from_str_radix(items[1].trim(),10)?,
                     u32::from_str_radix(items[2].trim(),10)?,
                     u32::from_str_radix(items[3].trim(),10)?,
                     u32::from_str_radix(items[4].trim(),10)?,
                     f32::from_str(items[5].trim())?,
-                    constellation::Constellation::from_str(items[6].trim())?);
+                    Constellation::from_str(items[6].trim())?);
                 let utc = chrono::NaiveDate::from_ymd(y,month,d).and_hms(h,min,s as u32);
-                epochs.1 = Some(GnssTime::new(utc, constel)) 
+                epochs.1 = Some(gnss_time::GnssTime::new(utc, constel)) 
             
             } else if line.contains("WAVELENGTH FACT L1/2") {
      //1     1                                                WAVELENGTH FACT L1/2
@@ -578,8 +546,14 @@ impl std::str::FromStr for Header {
                 coords = Some(rust_3d::Point3D::new(x,y,z))
 
             } else if line.contains("ANTENNA: DELTA H/E/N") {
-                //TODO
-        //0.0000        0.0000        0.0000                  ANTENNA: DELTA H/E/N
+                let (h, rem) = line.split_at(15);
+                let (e, rem) = rem.split_at(15);
+                let (n, _) = rem.split_at(15);
+                ant_hen = Some((
+                    f32::from_str(h.trim())?,
+                    f32::from_str(e.trim())?,
+                    f32::from_str(n.trim())?))
+
             } else if line.contains("ANTENNA: DELTA X/Y/Z") {
                 let items: Vec<&str> = line.split_ascii_whitespace()
                     .collect();
@@ -603,13 +577,20 @@ impl std::str::FromStr for Header {
                 rcvr_clock_offset_applied = Some(i32::from_str_radix(ok_str, 10)? != 0)
 
             } else if line.contains("# OF SATELLITES") {
-                //TODO
+                // will always appear prior PRN/#OBS
+                // determines nb of satellites in observation file
+                let (nb, _) = line.split_at(10);
+                obs_nb_sat = u32::from_str_radix(nb.trim(), 10)?
+
             } else if line.contains("PRN / # OF OBS") {
-                //TODO
-            } else if line.contains("SYS / PHASE SHIFT") {
-                //TODO
-            } else if line.contains("SYS / # / OBS TYPES") {
-                //TODO
+                let (sv, rem) = line.split_at(7);
+                if sv.trim().len() > 0 {
+                    
+                }
+                // lists all Sv
+                let items: Vec<&str> = line.split_ascii_whitespace()
+                    .collect();
+                 
             } else if line.contains("SYS / PHASE SHIFT") {
                 //TODO
             } else if line.contains("SYS / PVCS APPLIED") {
@@ -623,7 +604,44 @@ impl std::str::FromStr for Header {
                 // RINEX::ClockData specific 
                 // + number of different clock data types stored
                 // + list of clock data  types
-            } else if line.contains("# / TYPES OF OBSERV") {
+            } else if line.contains("TYPES OF OBS") {
+                // Rinex::ObservationData specific
+                // types of observation enumeration
+                // but it's not a per constellation enumeration
+                // => we set a flag and we will post process
+                //    to provide high level information in the end (sorting capacity)
+                //5    L1    L2    C1    P1    P2                        # / TYPES OF OBSERV
+                /*unspecified_obs_system = true;
+                let (num, rem) = line.split_at(6);
+                for i in 0..num {
+                    let (code, rem) = rem.split_at(6);
+                    if code.trim().len() == 0 {
+                        break
+                    }
+                    
+                }*/
+
+            } else if line.contains("SYS / # / OBS TYPES") {
+                // Rinex::ObservationData 
+                // types of observation for specified constellation
+                
+                // grab Constellation info:
+                //   [+] if constellation info exists: new entry
+                //   [+] otherwise: adding some more to a previous entry
+                let (sys, rem) = line.split_at(3);
+                let (nb, rem) = rem.split_at(3);
+                if sys.trim().len() > 0 {
+                    println!("SYS \"{}\" NB \"{}\" REM \"{}\"", sys.trim(), nb.trim(), rem.trim());
+                } else {
+                    println!("REM \"{}\"", rem.trim());
+                }
+//G   22 C1C L1C D1C S1C C1W S1W C2W L2W D2W S2W C2L L2L D2L  SYS / # / OBS TYPES
+//       S2L C5Q L5Q D5Q S5Q C1L L1L D1L S1L                  SYS / # / OBS TYPES
+//E   20 C1C L1C D1C S1C C6C L6C D6C S6C C5Q L5Q D5Q S5Q C7Q  SYS / # / OBS TYPES
+//       L7Q D7Q S7Q C8Q L8Q D8Q S8Q                          SYS / # / OBS TYPES
+//S    8 C1C L1C D1C S1C C5I L5I D5I S5I                      SYS / # / OBS TYPES
+            } else if line.contains("TYPES OF OBS") || 
+                line.contains("SYS / # / OBS TYPES") {
      //5    L1    L2    C1    P1    P2                        # / TYPES OF OBSERV
             
             } else if line.contains("SIGNAL STRENGHT UNIT") {
@@ -670,32 +688,52 @@ impl std::str::FromStr for Header {
             }
         }
         
-        Ok(Header{
+        if ant.is_some() { // whatever.. but will avoid hypothetical crash
+            if ant_coords.is_some() { // in case of faulty producer
+                ant.as_mut().unwrap().set_coords(ant_coords.unwrap())
+            }
+            if ant_hen.is_some() {
+                ant.as_mut().unwrap().set_height(ant_hen.unwrap().0);
+                ant.as_mut().unwrap().set_eastern_eccentricity(ant_hen.unwrap().1);
+                ant.as_mut().unwrap().set_northern_eccentricity(ant_hen.unwrap().2);
+            }
+        }
+        
+        Ok(RinexHeader{
             version: version,
             crinex: crinex_infos, 
             rinex_type,
             constellation,
             program: String::from(pgm.trim()),
-            run_by: run_by,
-            station: station,
-            station_id: station_id,
-            agency: agency,
-            observer: observer,
-            rcvr: rcvr, 
-            ant: ant, 
-            leap: leap,
-            rcvr_clock_offset_applied: rcvr_clock_offset_applied,
+            run_by,
+            station,
+            station_id,
+            agency,
+            observer,
+            license,
+            doi,
+            station_url,
+            rcvr, 
+            ant, 
+            leap,
             coords: coords,
+            // ?
             wavelengths: None,
-            sampling_interval: sampling_interval,
-            epochs: epochs,
             gps_utc_delta: None,
-            sat_number: None,
+            // observations
+            epochs: epochs,
+            obs_types: None,
+            // processing
+            sampling_interval: sampling_interval,
+            scaling_factor: None,
+            //ionospheric_corr: None,
+            //gnsstime_corr: None,
+            rcvr_clock_offset_applied,
         })
     }
 }
 
-impl Header {
+impl RinexHeader {
     /// Returns true if self is a `Compact RINEX`
     pub fn is_crinex (&self) -> bool { self.crinex.is_some() }
 
@@ -722,9 +760,12 @@ impl Header {
     }
 
     /// Returns `Rinex` Version number
-    pub fn get_rinex_version (&self) -> version::Version { self.version }
+    pub fn get_rinex_version (&self) -> RinexVersion { self.version }
     /// Returns `Rinex` type 
     pub fn get_rinex_type (&self) -> RinexType { self.rinex_type }
     /// Returns `GNSS` constellation
-    pub fn get_constellation (&self) -> constellation::Constellation { self.constellation }
+    pub fn get_constellation (&self) -> Constellation { self.constellation }
+
+    /// Returns `LeapSecond` infos (if any)
+    pub fn get_leap_second (&self) -> Option<LeapSecond> { self.leap }
 }
