@@ -1,90 +1,117 @@
 //! This package provides a set of tools to parse 
-//! and analyze RINEX files.
+//! `RINEX` files.
 //! 
+//! Refer to README for example of use.  
 //! Homepage: <https://github.com/gwbres/rinex>
+mod keys;
+mod header;
+mod version;
+mod gnss_time;
+mod navigation;
+mod observation;
+
+pub mod record;
+pub mod constellation;
 
 use thiserror::Error;
 use std::str::FromStr;
 use scan_fmt::scan_fmt;
+use std::collections::HashMap;
 
-mod keys;
-mod header;
-mod record;
-mod version;
-mod constellation;
+use header::RinexHeader;
+use record::{RinexRecord, RecordItem, RecordItemError};
+use version::RinexVersion;
+use constellation::Constellation;
 
 #[macro_export]
+/// Returns `true` if given `Rinex` line is a comment
 macro_rules! is_rinex_comment {
     ($line: expr) => { $line.contains("COMMENT") };
 }
 
+/// Describes all known `RINEX` file types
+#[derive(Copy, Clone, PartialEq, Debug)]
+pub enum RinexType {
+    /// Describes Observation Data (OBS),
+    /// Phase & Pseudo range measurements
+    ObservationData, 
+    /// Describes Navigation Message (NAV)
+    /// Ephemeride file
+    NavigationMessage,
+}
+
+#[derive(Error, Debug)]
+/// `RinexType` related errors
+pub enum RinexTypeError {
+    #[error("Unknown RINEX type identifier \"{0}\"")]
+    UnknownType(String),
+}
+
+impl Default for RinexType {
+    /// Builds a default `RinexType`
+    fn default() -> RinexType { RinexType::ObservationData }
+}
+
+impl RinexType {
+    /// Converts `Self` to str
+    pub fn to_str (&self) -> &str {
+        match *self {
+            RinexType::ObservationData => "ObservationData",
+            RinexType::NavigationMessage => "NavigationMessage",
+        }
+    }
+    /// Converts `Self` to string
+    pub fn to_string (&self) -> String { String::from(self.to_str()) }
+}
+
+impl std::str::FromStr for RinexType {
+    type Err = RinexTypeError;
+    fn from_str (s: &str) -> Result<Self, Self::Err> {
+        if s.eq("NAVIGATION DATA") {
+            Ok(RinexType::NavigationMessage)
+        } else if s.contains("NAV DATA") {
+            Ok(RinexType::NavigationMessage)
+        } else if s.eq("OBSERVATION DATA") {
+            Ok(RinexType::ObservationData)
+        } else {
+            Err(RinexTypeError::UnknownType(String::from(s)))
+        }
+    }
+}
+
 /// `Rinex` main structure,
 /// describes a `RINEX` file
 #[derive(Debug)]
 pub struct Rinex {
-    header: header::Header,
-    records: Vec<record::RinexRecord>,
+    header: RinexHeader,
+    record: RinexRecord,
 }
 
 impl Default for Rinex {
     /// Builds a default `RINEX`
     fn default() -> Rinex {
         Rinex {
-            header: header::Header::default(),
-            records: Vec::new(),
+            header: RinexHeader::default(),
+            record: Vec::new(),
         }
     }
 }
 
 #[derive(Error, Debug)]
+/// `RINEX` Parsing related errors
 pub enum RinexError {
     #[error("Header delimiter not found")]
     MissingHeaderDelimiter,
     #[error("Header parsing error")]
-    HeaderError(#[from] header::HeaderError),
+    HeaderError(#[from] header::Error),
 }
 
-/// macro to return true when a new block record
-/// has been identified
-pub fn new_record_block (line: &str,
-    rinex_type: &header::RinexType,
-        constellation: &constellation::Constellation, 
-            version: &version::Version) -> bool
-{
-    let parsed: Vec<&str> = line.split_ascii_whitespace()
-        .collect();
-    
-    match rinex_type {
-        header::RinexType::NavigationMessage => {
-            let known_sv_identifiers: &'static [char] = 
-                &['R','G','E','B','J','C','S']; 
-            match constellation {
-                constellation::Constellation::Glonass => parsed.len() > 4,
-                _ => {
-                    match line.chars().nth(0) {
-                        Some(c) => known_sv_identifiers.contains(&c), 
-                        _ => false
-                            //TODO
-                            // <o 
-                            //   for some files we end up with "\n xxxx" as first frame items 
-                            // current code will discard first payload item in such scenario
-                            // => need to cleanup (split(head,body) method)
-                    }
-                }
-            }
-        },
-        _ => false,
-    }
-}
-
-use record::*;
-
 impl Rinex {
-    /// Builds a Rinex struct
-    pub fn new (header: header::Header, records: Vec<record::RinexRecord>) -> Rinex {
+    /// Builds a new `RINEX` struct from given:
+    pub fn new (header: RinexHeader, record: RinexRecord) -> Rinex {
         Rinex {
             header,
-            records,
+            record,
         }
     }
 
@@ -105,8 +132,18 @@ impl Rinex {
     /// Returns `Rinex` length, ie.,
     ///   nb of observations for `RinexType::ObservationData`   
     ///   nb of ephemerides  for `RinexType::NavigationMessage`   
-    pub fn len (&self) -> usize { self.records.len() }
-    
+    pub fn len (&self) -> usize { self.record.len() }
+
+    /// Returns self's `header` section
+    pub fn get_header (&self) -> &RinexHeader { &self.header }
+
+    /// Returns entire RINEX record
+    pub fn get_record (&self) -> &RinexRecord { &self.record }
+
+    /// Returns Record nth' entry
+    pub fn get_record_nth (&self, nth: usize) 
+        -> &std::collections::HashMap<String, record::RecordItem> { &self.record[nth] }
+
     /// Builds a `Rinex` from given file.
     /// Input file must respect the whitespace specifications
     /// for the entire header section.   
@@ -120,19 +157,14 @@ impl Rinex {
             .unwrap();
 
         let (header, body) = Rinex::split_rinex_content(fp)?;
-        let header = header::Header::from_str(&header)?;
+        let header = RinexHeader::from_str(&header)?;
 
         // helpful information
         let rinex_type = header.get_rinex_type();
         let version = header.get_rinex_version();
         let version_major = version.get_major(); 
-        let version_minor = version.get_minor(); 
         let constellation = header.get_constellation();
 
-        // build key listing for this context
-        let keys = keys::KeyBank::new(&version, &rinex_type, &constellation)
-            .unwrap();
-
         let mut body = body.lines();
         let mut line = body.next()
             .unwrap(); // ''END OF HEADER'' /BLANK
@@ -145,32 +177,21 @@ impl Rinex {
         let mut eof = false;
         let mut first = true;
         let mut block = String::with_capacity(256*1024); // max. block size
-        let mut records: Vec<RinexRecord> = Vec::new();
+        let mut record: RinexRecord = Vec::new();
 
         loop {
             let parsed: Vec<&str> = line.split_ascii_whitespace()
                 .collect();
             
-            let new_block = new_record_block(&line, &rinex_type, &constellation, &version); 
-            if new_block && !first {
-                let record: Option<RinexRecord> = match rinex_type {
-                    header::RinexType::NavigationMessage => {
-                        if let Ok(record) = 
-                            navigation::NavigationRecord::from_string(version, constellation, &keys, &block) {
-                                Some(RinexRecord::RinexNavRecord(record))
-                        } else {
-                            None
-                        }
-                    },
-                    _ => None,
-                };
-                
-                if record.is_some() {
-                    records.push(record.unwrap())
+            let is_new_block = record::block_record_start(&line, &rinex_type, &constellation, &version); 
+            
+            if is_new_block && !first {
+                if let Ok(entry) = record::build_record_entry(version, rinex_type, constellation, &block) {
+                    record.push(entry)
                 }
             }
 
-            if new_block {
+            if is_new_block {
                 if first {
                     first = false
                 }
@@ -178,7 +199,7 @@ impl Rinex {
             }
 
             block.push_str(&line);
-            block.push_str(" ");
+            block.push_str("\n");
 
             if let Some(l) = body.next() {
                 line = l
@@ -202,7 +223,7 @@ impl Rinex {
 
         Ok(Rinex{
             header, 
-            records,
+            record,
         })
     }
 }
